package SecureComponent;

BEGIN
{
    my $CLIENT_MODULE_INCLUDE_PATH = './';

    @INC = (@INC, $CLIENT_MODULE_INCLUDE_PATH);
}

use HomebrewCryptor;



# For testing and measurement purposes only. Setting to '' will force HomebrewCryptor to prompt for the password.
my $SECURE_CONF_PASSWORD = 'ijk';

my $SECURE_CONF_FILE_PATH = 'config/secureConf.xml';

my $MAX_SECURE_CONF_DECRYPT_ATTEMPTS = 3;
# A newline will automatically be appended to the end of this message
my $FAILED_SECURE_CONF_DECRYPT_MESSAGE = "You entered the wrong password. Please try again.";
my $SECURE_CONF_DECRYPT_MESSAGE = "Please enter you secureConf password: ";

my $PRINT_INFORMATION = 0;
my $PRINT_ERROR = 1;
my $PRINT_WARNING = 1;

my $MODULE_NAME = 'SecureComponent';
my $PRINT_INFORMATION_PREFIX = 'INFORMATION - ' . $MODULE_NAME . ' - ';
my $PRINT_ERROR_PREFIX = 'ERROR - ' . $MODULE_NAME . ' - ';
my $PRINT_WARNING_PREFIX = 'WARNING - ' . $MODULE_NAME . ' - ';



my $ID_PROVIDER_INFORMATION_ELEMENT_NAME = 'IDProviderInformation';
my $CREDIT_CARD_INFORMATION_ELEMENT_NAME = 'creditCardInformation';
my $CREDIT_CARD_NAME_ELEMENT_NAME = 'creditCardName';
my $DOMAIN_ELEMENT_NAME = 'domain';

my $NAME_ATTRIBUTE_NAME = 'name';


my $CACHED_SECURE_CONF_PARSE_TREE_KEY = 'secureConfParseTree';
my $CACHED_ID_PROVIDER_INFORMATION_ELEMENT_KEY = 'cachedIDProviderInformation';
my $CACHED_CREDIT_CARD_INFORMATION_ELEMENT_KEY = 'cachedCreditCardInformation';

my $CRYPTOR_PROMPT_KEY = 'prompt';
my $CRYPTOR_PASSWORD_KEY = 'password';



my $LOGIN_INFO_USER_INFO_TYPE = 'loginInfo';
my $CREDIT_CARD_USER_INFO_TYPE = 'creditCard';



my $TRUE = 1;
my $FALSE = 0;

my $NORMAL_RETURN_STATUS = $TRUE;
my $ERROR_RETURN_STATUS = $FALSE;



my $returnStatus;


sub new
{
    my $class = shift;

    my $this = { };
    bless($this, $class);

    return $this;
}



sub initSecureConf
{
    my $PRINT_LOCAL_PREFIX = 'initSecureConf() - ';

    my $this = shift;
    my ($selectedIDProviderDomain) = @_;

    # Parse the secure conf file
    if (-r $SECURE_CONF_FILE_PATH)
    {
	if ($PRINT_INFORMATION)
	{
	    print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX .
		  "Decrypting and parsing secureConf\n");
	}
	
	my $secureConfParser = XML::LibXML->new();
	my $secureConfXML;
	
	my $decryptAttemptCount = 0;
	my $keepTryingToDecrypt = 1;
	
	while ($keepTryingToDecrypt)
	{
	    $decryptAttemptCount ++;
	    
	    #TODO - Maybe explain the whole CRYPTOR_PASSWORD_KEY thing
	    my $secureConfDecryptor = HomebrewCryptor
		->new($CRYPTOR_PROMPT_KEY => $SECURE_CONF_DECRYPT_MESSAGE,
		      $CRYPTOR_PASSWORD_KEY => $SECURE_CONF_PASSWORD);
	    
	    $secureConfXML = $secureConfDecryptor->decryptDataFromFile($SECURE_CONF_FILE_PATH);
	    
	    eval { $this->{$CACHED_SECURE_CONF_PARSE_TREE_KEY} = $secureConfParser->parse_string($secureConfXML) };
	    
	    #TODO - Could do XML validation here.
	    
	    # We stop trying to decrypt when either there are no errors from parsing the decrypted
	    #   file (Meaning decryption was successful) or when the user has tried too many times
	    if (!($@))
	    {
		$keepTryingToDecrypt = $FALSE;
	    }
	    elsif ($decryptAttemptCount == $MAX_SECURE_CONF_DECRYPT_ATTEMPTS)
	    {
		if ($PRINT_ERROR)
		{
		    print(STDERR $PRINT_ERROR_PREFIX . $PRINT_LOCAL_PREFIX .
			  "Too many decrypt attempts.\n");
		}
		
		return $ERROR_RETURN_STATUS;
	    }
	    else
	    {
		print($FAILED_SECURE_CONF_DECRYPT_MESSAGE . "\n");
	    }
	}
	
	if ($PRINT_INFORMATION)
	{
	    print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX . "Secure conf = \n");
	    print(STDERR $secureConfXML);
	}
    }
    else
    {
	if ($PRINT_ERROR)
	{
	    print(STDERR $PRINT_ERROR_PREFIX . $PRINT_LOCAL_PREFIX . "Cant find secureConf file\n");
	}
	
	return $ERROR_RETURN_STATUS;
    }

    return $NORMAL_RETURN_STATUS;
}

sub cacheSecureConfElements
{
    my $this = shift;

    my ($selectedIDProviderDomain) = @_;

    my $secureConfParseTree = $this->{$CACHED_SECURE_CONF_PARSE_TREE_KEY};
    # Get all of the ID provider and credit card elements
    my @IDProviderElements = $secureConfParseTree->getElementsByTagName($ID_PROVIDER_INFORMATION_ELEMENT_NAME);
    my @creditCardElements = $secureConfParseTree->getElementsByTagName($CREDIT_CARD_INFORMATION_ELEMENT_NAME);

    # Choose the appropriate (Selected) ID provider and corresponding credit card elements.
    foreach my $currentIDProviderElement (@IDProviderElements)
    {
	my ($currentIDProviderDomainElement) =
	    $currentIDProviderElement->getChildrenByTagName($DOMAIN_ELEMENT_NAME);

	if ($currentIDProviderDomainElement->textContent eq $selectedIDProviderDomain)
	{
	    $this->{$CACHED_ID_PROVIDER_INFORMATION_ELEMENT_KEY} = $currentIDProviderElement;
	    
	    my ($associatedCreditCardElement) = $currentIDProviderElement
		->getChildrenByTagName($CREDIT_CARD_NAME_ELEMENT_NAME);

	    my $associatedCreditCardName = $associatedCreditCardElement->textContent;

	    foreach my $currentCreditCardElement (@creditCardElements)
	    {
		if ($currentCreditCardElement->getAttribute($NAME_ATTRIBUTE_NAME)
		    eq $associatedCreditCardName)
		{
		    $this->{$CACHED_CREDIT_CARD_INFORMATION_ELEMENT_KEY} = $currentCreditCardElement;
		}
	    }
	}
    }    
}

sub lookupSecureInfo
{
    my $PRINT_LOCAL_PREFIX = 'lookupSecureInfo() - ';

    my $this = shift;

    my ($desiredInfoDesignatorName, $selectedIDProviderDomain) = @_;

    my ($desiredUserInfoType, $desiredUserInfoName) = split(/\//, $desiredInfoDesignatorName);



    if ($PRINT_INFORMATION)
    {
	print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX .
	      "Looking for info designator \'" . $desiredInfoDesignatorName . "\'.\n");
    }

    if (!($this->{$CACHED_SECURE_CONF_PARSE_TREE_KEY}))
    {
	if ($PRINT_INFORMATION)
	{
	    print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX .
		  "Initializing secureConf (Decrypting file, parsing).\n");
	}

	$returnStatus = $this->initSecureConf($selectedIDProviderDomain);

	if (!($returnStatus))
	{
	    #TODO - Print a warning. This method wont really return a bad return status, so theres not much we can do there.
	}
    }
    else
    {
	if ($PRINT_INFORMATION)
	{
	    print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX .
		  "Using cached secureConf parse tree.\n");
	}
    }

    if (!($this->cachedSecureConfElementsAreValid($selectedIDProviderDomain)))
    {
	if ($PRINT_INFORMATION)
	{
	    print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX .
		  "Getting new secureConf elements.\n");
	}

	$this->cacheSecureConfElements($selectedIDProviderDomain);
    }
    else
    {
	if ($PRINT_INFORMATION)
	{
	    print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX .
		  "Using cached secureConf elements.\n");
	}
    }
    
    #TODO - This is a temporary translation to be used until the new UserInfoDesignator elements are put in place.
    if ($desiredUserInfoType eq "login_info")
    {
        $desiredUserInfoType = "loginInfo";
    }
    if ($desiredUserInfoType eq "credit_card")
    {
        $desiredUserInfoType = "creditCard";
    }
    
    if ($desiredUserInfoName eq "user_id")
    {
        $desiredUserInfoName = "userName";
    }
    if ($desiredUserInfoName eq "card_number")
    {
        $desiredUserInfoName = "number";
    }

    my $selectedIDProviderElement = $this->{$CACHED_ID_PROVIDER_INFORMATION_ELEMENT_KEY};
    my $selectedCreditCardElement = $this->{$CACHED_CREDIT_CARD_INFORMATION_ELEMENT_KEY};

    my $associatedUserInfoElement;

    if ($desiredUserInfoType eq $LOGIN_INFO_USER_INFO_TYPE)
    {
	if ($selectedIDProviderElement)
	{
	    ($associatedUserInfoElement) = $selectedIDProviderElement->getChildrenByTagName($desiredUserInfoName);
	}
	else
	{
	    if ($PRINT_WARNING)
	    {
		print(STDERR $PRINT_WARNING_PREFIX . $PRINT_LOCAL_PREFIX .
		      "ID Provider undefined (Not found in secure conf)\n");
	    }
	}
    }
    elsif ($desiredUserInfoType eq $CREDIT_CARD_USER_INFO_TYPE)
    {
	if ($selectedCreditCardElement)
	{
	    ($associatedUserInfoElement) = $selectedCreditCardElement->getChildrenByTagName($desiredUserInfoName);
	}
	else
	{
	    if ($PRINT_WARNING)
	    {
		print(STDERR $PRINT_WARNING_PREFIX . $PRINT_LOCAL_PREFIX .
		      "Credit card undefined (Not found in secure conf)\n");
	    }
	}
    }
    else
    {
	if ($PRINT_WARNING)
	{
	    print(STDERR $PRINT_WARNING_PREFIX . $PRINT_LOCAL_PREFIX .
		  "Unknown user info type " . $desiredUserInfoType . "\n");
	}
    }

    if ($associatedUserInfoElement)
    {
	if ($PRINT_INFORMATION)
	{
	    print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX . "Associated info = " .
		  $associatedUserInfoElement->textContent . "\n");
	}
	
	return ($associatedUserInfoElement->textContent);
    }
    else
    {
	if ($PRINT_WARNING)
	{
	    print(STDERR $PRINT_WARNING_PREFIX . $PRINT_LOCAL_PREFIX .
		  "Information not found in secureConf\n");
	}
    }

    return "";
}

sub cachedSecureConfElementsAreValid
{
    my $PRINT_LOCAL_PREFIX = 'cachedSecureConfElementsAreValid() - ';

    my $this = shift;
    
    my ($currentIDProviderDomain) = @_;

    my $cachedIDProviderInformationElement = $this->{$CACHED_ID_PROVIDER_INFORMATION_ELEMENT_KEY};

    if ($cachedIDProviderInformationElement)
    {
	#TODO - We will assume that this is not null
	my ($cachedIDProviderDomainElement) = $cachedIDProviderInformationElement
	    ->getChildrenByTagName($DOMAIN_ELEMENT_NAME);
	my $cachedIDProviderDomain = $cachedIDProviderDomainElement->textContent;

	if ($PRINT_INFORMATION)
	{
	    print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX .
		  "Comparing current ID Provider Domain = \'" . $currentIDProviderDomain .
		  "\' and cached ID Provider Domain = \'" . $cachedIDProviderDomain . "\'.\n");
	}

	if ($currentIDProviderDomain eq $cachedIDProviderDomain)
	{
	    return $TRUE;
	}
	else
	{
	    return $FALSE;
	}
    }
    else
    {
	if ($PRINT_INFORMATION)
	{
	    print(STDERR $PRINT_INFORMATION_PREFIX . $PRINT_LOCAL_PREFIX .
		  "No cached secureConf elements.\n");
	}

	return $FALSE;
    }
}



return $TRUE;
