#!/usr/bin/perl -w
#(get_l2_info.pl)
#Retrieves the L2 information required for compound L2-web authentication
#Author: Ana Sanz Merino [University of California, Berkeley - 2005]
#E-mail: asanz@eecs.berkeley.edu
#Revision History
#ver.0.1 01/21/05

use Time::HiRes qw(usleep ualarm gettimeofday tv_interval);
use strict;
use IO::Socket;
use Digest::MD5 qw(md5_hex);

#Contants
my $server_sock_dir = "/tmp/socket";
my $server_sock_buf_size = 150;
my $linux = 1;                  #for testing

#Variables
my $wlan_ifc;
my $server_sock;
my $l2_key;
my $mac_addr;
my $l2_key_digest;

#Main
$wlan_ifc = &GetWlanInterface(@ARGV);
$mac_addr = &GetMacAddr($wlan_ifc);
$l2_key = &GetL2SessionKey();
$l2_key_digest = &GetKeyDigest($l2_key);
print ($l2_key_digest.' '.$mac_addr);

################################################################################
### GetWlanInterface (@argv)
### Get WLAN interface from command line argument
### Input: @argv - command line argument
### Return: $wlan_ifc
################################################################################
sub GetWlanInterface {
    print (STDERR "Getting wireless interface...\n");
    my @argv = $_[0];
    my $argc = @argv;
    my $ifc;

    if ($argc != 1) {
        die "Usage: get_l2_info.pl <wlan interface>\n";
    }

    $ifc = $argv[0];
    return ($ifc);
}

       
################################################################################
### GetL2SessionKey()
### Retrieve the session key that xsupplicant outputs
### Returns: 802.1x session key (result migth not be readable)
################################################################################
sub GetL2SessionKey {
    my $buf;
    my (@msg, $i);
    my ($hex_session_key, $session_key);

    #Open socket to receive the session key info sent by xsupplicant
    unlink $server_sock_dir;
    $server_sock = IO::Socket::UNIX->new(Local => $server_sock_dir,
                                         Type => SOCK_DGRAM,
                                         Listen => 5)
        or die "Could not open server socket\n";

    while (defined (recv($server_sock, $buf, $server_sock_buf_size, 0))) {
        @msg = split(" ", $buf);
        if ($msg[0] eq  'RECV_SESSION_KEY') {
            print (STDERR "Session Key Received.\n");
            if ( ($#msg < 1) || (length($msg[1])<64)  ){
                printf (STDERR "Error: Session Key Length too short.\n");
                return "ERROR";
            }
            $hex_session_key = $msg[1];
            printf(STDERR "TLS session key (hex) = %s\n", $hex_session_key);
            $session_key = &HexToStr($hex_session_key);
            last;
        } elsif ($msg[0] eq  'SEND_EAPOL_START') {
            print (STDERR "Sent EAPOL_START, Waiting for ID Request.\n");
        } elsif ($msg[0] eq 'RECV_ID_REQ') {
            print (STDERR "ID Request Received, Sending ID Response.\n");
        } elsif ($msg[0] eq 'RESEND_ID_RSP') {
            print (STDERR "Resending ID Response.\n");
        } elsif ($msg[0] eq 'RECV_EAP_AUTH_REQ') {
            print (STDERR "EAP Auth Req Received, Sending Rsp.\n");
        } elsif ($msg[0] eq 'RESEND_EAP_AUTH_RSP') {
            print (STDERR "Resending EAP Auth Rsp.\n");
         } elsif ($msg[0] eq 'SEND_EAPOL_LOGOFF') {
            print (STDERR "Sent EAPOL_LOGOFF\n");
        } elsif ($msg[0] eq 'RECV_EAP_SUCCESS') {
            print (STDERR "EAP Success Received.\n");
        } elsif ($msg[0] eq 'RECV_EAP_FAILURE') {
            print (STDERR "EAP Failure Received.\n");
            return "ERROR";
        } else {
            print (STDERR "Unrecognizable message received from XSupplicant\n");
            printf(STDERR "msg = %s\n", $msg[0]);
            return "ERROR";
        }
    }

    return $session_key;
}


################################################################################
### GetMacAddr($wlan_ifc)
### Get Current MAC Address from ifconfig output
### Input: $wlan_ifc - wireless LAN interface name to be queried
### Returns: MAC Address
################################################################################
sub GetMacAddr {
    my ($ifc) = @_;
    my @buf;            #ifconfig result buffer
    my ($line, @words, $element);
    my $mac = '';
    my $prev_element = "";

    if ($linux) {
        @buf = `/sbin/ifconfig $ifc`;
    }

    foreach $line (@buf) {
        chomp $line;
        @words = split(/\s/, $line);
        foreach $element (@words) {
            if ($prev_element =~ /HWaddr/) {
                $mac = $element;
            }
            $prev_element = $element;
        }
    }

    if ($mac eq '') {
        die "Error retrieving MAC address\n";
    }

    print(STDERR "GetMacAddr: mac = $mac\n");

    return $mac;
}


################################################################################
### GetKeyDigest($key)
### Calculate MD5 digest
### Input: $key - key which calculate the digest from
### Returns: MD5 digest
################################################################################
sub GetKeyDigest {
    my ($in_key) = @_;
    my $digest;

    $digest = md5_hex($in_key);
    print(STDERR "GetKeyDigest: digest = $digest\n");

    return $digest;
}
                                                                                                
################################################################################
### HexToStr($hex_str)
### Converts an hexadecimal representation of bytes to the bytes themselves
### Input: $hexStr - string containing the hexadecimal representation of the
###                  bytes
### Returns: the bytes
################################################################################
sub HexToStr {
    my ($hex_str) = @_;
    my $str;

    $str=pack('H*', $hex_str);

    return $str;
}
