/*
 * $Id: SimpleAuthnRequestProcessor.java,v 1.22.2.1 2002/09/17 01:52:26 jsergent Exp $
 * SimpleAuthnRequestProcessor.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): Ana Sanz Merino [University of California Berkeley, 2003]
 */

package com.sun.liberty;
import com.sun.liberty.saml.SAMLAudienceRestrictionCondition;
import com.sun.liberty.saml.SAMLConditions;
import com.sun.liberty.saml.SAMLElement;
import com.sun.liberty.saml.SAMLNameIdentifier;
import com.sun.liberty.saml.SAMLSubjectConfirmation;
import com.sun.liberty.saml.SAMLPStatus;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.security.SecureRandom;

/**
 * Processes incoming AuthnRequest messages.  Does not do anything
 * fancy with authentication context or session index.  Always obtains
 * user consent for federation.
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.22.2.1 $
 */

public class SimpleAuthnRequestProcessor implements AuthnRequestProcessor {
    private static final String NAME_FORMAT = null;
    private static final String NAME_DOMAIN = null;
    private static ProviderDescriptorFactory dir;
    private static SecureRandom random = new SecureRandom();

    /**
     * Simple implementation for processing AuthnRequest messages.
     * Does not support attempting to upgrade authentication context
     * to comply with service provider requests.
     */
    public RequestProcessorAction process(LibertyAuthnRequest request,
					  LibertyAuthnResponse response,
					  boolean userAuthAttempted,
					  boolean consentObtained,
					  LibertyUserSession session)
	throws Exception
    {
	if (dir == null) {
	    dir = ProviderDescriptorFactory.getInstance();
	}

	IDPDescriptor us;
	us = (IDPDescriptor) dir.findLocal();
	SPDescriptor sp = null;

	String failureCode = null;

	/*
	 * Check request validity before going any further
	 */
	long offset = new Date().getTime();
	offset -= request.getIssueInstant().getTime();
	offset = Math.abs(offset);
	if (request.getMajorVersion() != 1) {
	    if (Config.debug) {
		System.err.println("rejected due to major version mismatch");
	    }
	    failureCode = "samlp:VersionMismatch";
	} else if (offset > Config.issueInstantTolerance) {
	    if (Config.debug) {
		System.err.println("rejected request due to issue instant");
		System.err.println("offset:" + offset);
		System.err.println("tolerance:" + Config.issueInstantTolerance);
	    }
	    failureCode = "samlp:Sender";
	} else {
	    sp = (SPDescriptor) dir.find(request.getProviderID());
	    if (sp == null) {
		failureCode = "samlp:Sender";
		if (Config.debug) { 
		    System.err.println("rejected request due to provider ID");
		    System.err.println("providerID:" + request.getProviderID());
		}
	    } else if (sp.getAuthnRequestsSigned() == true) {
	       if (request.getSignedBy() == null
		   || !request.getSignedBy().equals(request.getProviderID()))
	       {
		   if (Config.debug) {
		       System.err.println("AuthnRequest needs signature");
		   }
		   failureCode = "samlp:Sender";
	       }
	    }
	}

	if (session == null) {
	    if (Config.debug) { System.err.println("session is null!"); }
	}

	LibertyUser user = session.getUser();

	if (user == null || (request.getForceAuthn() && !userAuthAttempted)) {
	    // Authentication needed to succesfully process request
	    if (request.getIsPassive() || userAuthAttempted) {
		// Authentication has already failed or is being inhibited
		if (Config.debug) {
		    System.err.println("not authned -- passive or attempted");
		}
		failureCode = "samlp:Sender";
	    }
	    else {
		if (Config.debug) {
		    System.err.println("need to attempt user auth...");
		}
		return RequestProcessorAction.ATTEMPT_USER_AUTH;
	    }
	}
	else {
	    boolean wasFederated = user.getFederationStatus(sp);

	    if (wasFederated == false) {
		if (Config.debug) {
		    System.err.println("wasFederated == false");
		}
		if (!request.getFederate()) {
		    if (Config.debug) {
			System.err.println("getFederate() == false");
		    }
		    failureCode = "samlp:Sender";
		} else {
		    /* Federate user -- must obtain consent */
		    if (consentObtained == false) {
			return RequestProcessorAction.OBTAIN_USER_CONSENT;
		    } else {
			/* Time to federate */
			SAMLNameIdentifier newname = new SAMLNameIdentifier();
			newname.setNameQualifier(NAME_DOMAIN);
			newname.setFormat(NAME_FORMAT);
			byte[] namebytes = new byte[24];
			random.nextBytes(namebytes);
			newname.setValue(URLUtils.base64Encode(namebytes));
			user.setFederationStatus(sp, true);
			user.setLocalNameFor(sp, newname);
			/* Unless and until we get a register name request: */
			user.setNameProvidedBy(sp, newname);
		    }
		}
	    }
	}

	if (Config.debug) {
	    System.err.println("constructing AuthnResponse...");
	}

	response.setInResponseTo(request.getRequestID());
	response.setRecipient(request.getProviderID());
	response.setRelayState(request.getRelayState());

	SAMLPStatus status = new SAMLPStatus();
	if (failureCode != null) {
	    status.setStatusCode(failureCode);
	    response.setStatus(status);
	    if (Config.debug) {
		System.err.println("returning failureCode:" + failureCode);
	    }
	    return RequestProcessorAction.RETURN_RESPONSE;
	} else {
	    if (Config.debug) {
		System.err.println("status code is samlp:Success");
	    }
	    status.setStatusCode("samlp:Success");
	    response.setStatus(status);
	}

	/* invariant: user.getFederationStatus(sp) == true */

	/* User is authenticated correctly -- return proper assertion */
	
	LibertyAssertion assertion = new LibertyAssertion();
	List aset = new LinkedList();
	aset.add(assertion);
	response.setAssertions(aset);

	/* assertion: top level attrs */
	assertion.setIssuer(us.getProviderID());
	assertion.setInResponseTo(response.getInResponseTo());

	/* conditions: date and time */
	SAMLConditions conditions = new SAMLConditions();
	assertion.setConditions(conditions);
	long now = new Date().getTime();
	conditions.setNotBefore(new Date(now - Config.answerInstantTolerancePre));
	conditions.setNotOnOrAfter(new Date(now + Config.answerInstantTolerancePost));

	/* conditions: audience restriction condition */
	SAMLAudienceRestrictionCondition arc;
	arc = new SAMLAudienceRestrictionCondition();
	List audienceList = new LinkedList();
	audienceList.add(sp.getProviderID());
	arc.setAudiences(audienceList);
	List conditionList = new LinkedList();
	conditionList.add(arc);
	conditions.setConditions(conditionList);

	/* authentication statement */
	LibertyAuthenticationStatement stmt = new LibertyAuthenticationStatement();
	List sset = new LinkedList();
	sset.add(stmt);
	assertion.setStatements(sset);

	stmt.setAuthenticationInstant(new Date());
	// Not filling in stmt auth method, subject locality, or authority bindings
	
	LibertySubject subj = new LibertySubject();
	stmt.setSubject(subj);

	try {
	    subj.setNameIdentifier(user.getNameProvidedBy(sp));
	    LibertyIDPProvidedNameIdentifier idpName;
	    idpName = new LibertyIDPProvidedNameIdentifier();
	    idpName.parse(user.getLocalNameFor(sp).toString());
	    subj.setIDPProvidedNameIdentifier(idpName);
	} catch (Exception e) {
	    /* invariant: shouldn't happen (due to above invariant) */
	    e.printStackTrace();
	}

	SAMLSubjectConfirmation conf = new SAMLSubjectConfirmation();
	subj.setSubjectConfirmation(conf);
	List confirmationMethods = new LinkedList();
	LibertyProfile profile = request.getProtocolProfile();
	if (profile == LibertyProfile.BRWS_ART) {
	    confirmationMethods.add(SAMLElement.CM_ARTIFACT);
	} else if (profile == LibertyProfile.BRWS_POST
	           || profile == LibertyProfile.WML_POST) {
	    confirmationMethods.add(SAMLElement.CM_BEARER);
	}
	/*
	 * XXX: For LECP, we need to set conf method based on how we authenticated
	 * the user.
	 */
	conf.setConfirmationMethods(confirmationMethods);
	
	String authprofile = session.getAuthnContextProfileRef();

	if (authprofile != null) {
	    /* FIXME: need to do this right 
	    response.setAuthenticationContextProfileRef(authprofile);
	    */
	}

	assertion.setSignedBy(us.getProviderID());
	session.addRelyingSP(sp);
	if (Config.debug) {
	    System.err.println("returning successful response");
	}
	return RequestProcessorAction.RETURN_RESPONSE;
    }
}
