/*
 * $Id: XMLProviderDescriptorFactory.java,v 1.3.2.1 2002/09/17 01:52:28 jsergent Exp $
 * ProviderDescriptorFactory.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty;
import java.net.URL;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.List;
import java.util.LinkedList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.apache.xml.security.keys.KeyInfo;

/**
 * Implements a provider descriptor factory by parsing an XML file containing
 * a series of SPDescriptor and IDPDescriptor elements.
 */

public class XMLProviderDescriptorFactory extends ProviderDescriptorFactory {
    private class XMLProviderDescriptor extends SimpleProviderDescriptor {
        XMLProviderDescriptor(Element de) throws Exception {
            for (Node n = de.getFirstChild(); n != null;
                 n = n.getNextSibling()) {
                if (!(n instanceof Element)) { continue; }
                Element e = (Element) n;
                String t = e.getFirstChild().getNodeValue();
                if (XMLUtils.isElement(e, LibertyElement.LIB_NS_URI,
                                       "ProviderID")) {
                    providerID = t;
                } else if (XMLUtils.isElement(e, LibertyElement.LIB_NS_URI,
                                              "ProviderSuccinctID")) {
                    providerSuccinctID = new byte[t.length()/2];
                    for (int i = 0, j = 0; i < t.length(); i += 2, j++) {
			providerSuccinctID[j] = (byte) 
			    ((Character.digit(t.charAt(i), 16) << 4)
			    | (Character.digit(t.charAt(i+1), 16)));
                    }
                    b64ProviderSuccinctID =
                        URLUtils.base64Encode(providerSuccinctID);
		    if (Config.debug) {
			System.err.println("hex:" + t + " b64:"
					   + b64ProviderSuccinctID);
		    }
                } else if (XMLUtils.isElement(e, XMLElement.DSIG_NS_URI,
                                              "KeyInfo")) {
                    keyInfo = new KeyInfo(e, "");
                } else if (XMLUtils.isElement(e, LibertyElement.LIB_NS_URI,
                                              "SoapEndpoint")) {
                    soapEndpoint = new URL(t);
                } else if (XMLUtils.isElement(e, LibertyElement.LIB_NS_URI,
                                              "SingleLogoutServiceURL")) {
                    singleLogoutURL = new URL(t);
                } else if (XMLUtils.isElement(e, LibertyElement.LIB_NS_URI,
                                              "SingleLogoutServiceReturnURL")) {
                    singleLogoutReturnURL = new URL(t);
                } else if (XMLUtils.isElement(e, LibertyElement.LIB_NS_URI,
                                        "FederationTerminationServiceURL")) {
                    federationTerminationURL = new URL(t);
                } else if (XMLUtils.isElement(e, LibertyElement.LIB_NS_URI,
                                    "FederationTerminationServiceReturnURL")) {
                    federationTerminationReturnURL = new URL(t);
                } 
            }
        }
    }

    private class XMLSPDescriptor extends XMLProviderDescriptor implements SPDescriptor {
        URL assertionConsumerURL = null;
        LibertyProfile federationTerminationProtocolProfile = null;
        LibertyProfile singleLogoutProtocolProfile = null;
        boolean authnRequestsSigned;

        XMLSPDescriptor(Element de) throws Exception {
            super(de);
            for (Node n = de.getFirstChild(); n != null;
                 n = n.getNextSibling()) {
                if (!(n instanceof Element)) { continue; }
                Element e = (Element) n;
                String t = e.getFirstChild().getNodeValue();
                if (XMLUtils.isElement(e, LibertyElement.LIB_NS_URI,
                                       "AssertionConsumerServiceURL")) {
                    assertionConsumerURL = new URL(t);
                } else if (XMLUtils.isElement(e, LibertyElement.LIB_NS_URI,
                        "FederationTerminationNotificationProtocolProfile")) {
                    federationTerminationProtocolProfile = LibertyProfile.valueOf(t);
                } else if (XMLUtils.isElement(e, LibertyElement.LIB_NS_URI,
                                              "SingleLogoutProtocolProfile")) {
                    singleLogoutProtocolProfile = LibertyProfile.valueOf(t);
                } else if (XMLUtils.isElement(e, LibertyElement.LIB_NS_URI,
                                              "AuthnRequestsSigned")) {
                    authnRequestsSigned = t.equals("true") || t.equals("1");
                }
            }
        }
        public URL getAssertionConsumerURL() { return assertionConsumerURL; }

        public LibertyProfile getFederationTerminationProtocolProfile() {
            return federationTerminationProtocolProfile;
        }

        public LibertyProfile getSingleLogoutProtocolProfile() {
            return singleLogoutProtocolProfile;
        }

        public boolean getAuthnRequestsSigned() { return authnRequestsSigned; }
    }

    private class XMLIDPDescriptor extends XMLProviderDescriptor implements IDPDescriptor {
        URL singleSignOnURL = null;

        XMLIDPDescriptor(Element de) throws Exception {
            super(de);
            for (Node n = de.getFirstChild(); n != null;
                 n = n.getNextSibling()) {
                if (!(n instanceof Element)) { continue; }
                Element e = (Element) n;
                String t = e.getFirstChild().getNodeValue();
                if (XMLUtils.isElement(e, LibertyElement.LIB_NS_URI,
                                       "SingleSignOnServiceURL")) {
                    singleSignOnURL = new URL(t);
                }
            }
        }

        public URL getSingleSignOnURL() { return singleSignOnURL; }
    }
                    

    Map idToDescriptor = new HashMap();
    Map succinctIDToDescriptor = new HashMap();
    LinkedList identityProviders = new LinkedList();
    LinkedList serviceProviders = new LinkedList();

    /**
     * Parses the configured XML file to create a new
     * XMLProviderDescriptorFactory.
     */
    public XMLProviderDescriptorFactory() throws Exception {
        Document doc = XMLUtils.parseFile(Config.xmlProviderDescriptorFile);
        Element dir = doc.getDocumentElement();
        if (dir.getTagName().compareToIgnoreCase("providerdirectory") != 0) {
            throw new Exception("ProviderDirectory element not found");
        }
        for (Node n = dir.getFirstChild(); n != null; n = n.getNextSibling()) {
            if (!(n instanceof Element)) { continue; }
            Element e = (Element) n;
            ProviderDescriptor d;
            if (XMLUtils.isElement(e, LibertyElement.LIB_NS_URI,
                                   "SPDescriptor")) {
                d = new XMLSPDescriptor(e);
                serviceProviders.add(d);
            } else if (XMLUtils.isElement(e, LibertyElement.LIB_NS_URI,
                                          "IDPDescriptor")) {
                d = new XMLIDPDescriptor(e);
                identityProviders.add(d);
            } else {
                d = new XMLProviderDescriptor(e);
            }
            idToDescriptor.put(d.getProviderID(), d);
            succinctIDToDescriptor.put(new String(d.getProviderSuccinctID()),
                                       d);
        }
    }
    
    public ProviderDescriptor findLocal() {
        return find(Config.localProviderID);
    }

    public ProviderDescriptor find(String providerID) {
        return (ProviderDescriptor) idToDescriptor.get(providerID);
    }

    public ProviderDescriptor find(byte[] providerSuccinctID) {
        return (ProviderDescriptor) succinctIDToDescriptor.get(
                                            new String(providerSuccinctID));
    }

    public List getSPList() {
        return Collections.unmodifiableList(serviceProviders);
    }

    public List getIDPList() {
        return Collections.unmodifiableList(identityProviders);
    }
}
