package ucb.eecs.sahara.sso.anp;

import ucb.eecs.sahara.sso.DebugUtils;

import com.sun.liberty.Config;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.sun.liberty.saml.SAMLPStatus;
import com.sun.liberty.ProviderDescriptorFactory;
import com.sun.liberty.ProviderDescriptor;

import java.util.LinkedList;
import java.util.List;
import java.util.Iterator;


/**
 * @author Ana Sanz Merino
 * @version $Revision: 1.1.1.1 $
 */

public class SimpleMessageANPRequestProcessor implements MessageANPRequestProcessor {    

    private static ProviderDescriptorFactory dir;

    public ANPRequestProcessorAction process(HttpServletRequest httpReq,
                                             HttpServletResponse httpResp,
					     ANPRequest request,
					     ANPResponse response) 
    {
        DebugUtils.debugTime("SimpleMessageANPRequestProcessor", "Entering process",
                             DebugUtils.LOW_LEVEL_FUNC);
        try {
            ANPRequestProcessor proc = new SimpleANPRequestProcessor();
            ANPRequestProcResult procResult = new ANPRequestProcResult();
            ANPRequestProcessorAction action = null;
            action = proc.process(httpReq, httpResp, request, procResult);

            if (action == ANPRequestProcessorAction.RETURN_RESPONSE) {
                if (procResult.failureCode != null) {
                    fillBasicResponse(procResult.failureCode, request,
                                      response);
                    return ANPRequestProcessorAction.RETURN_RESPONSE;
                } else {
                    if (procResult.queryType.equals(
                        ANPRequestProcResult.AUTHN_CAPB_QUERY)) {
                        return generateANPAuthnCapabilitiesResponse(request,
                                                                    response,
                                                                    procResult);
                    } else if (procResult.queryType.equals(
                        ANPRequestProcResult.AUTHN_QUERY)) {
                        return generateANPAuthnResponse(request, response,
                                                        procResult);
                    } else {
                        return ANPRequestProcessorAction.FAILED;
                    }
                }
            } else if (action == ANPRequestProcessorAction.FAILED) {
                return action;
            } else {
		/* Got an invalid ANPRequestProcessorAction value */
                return ANPRequestProcessorAction.FAILED;
            }
        } catch (Exception e) {
            if (Config.debug) {
                System.err.println(e);
            }
            return ANPRequestProcessorAction.FAILED;
        }
    }


    private ANPRequestProcessorAction generateANPAuthnCapabilitiesResponse(
                                               ANPRequest request,
					       ANPResponse response,
                                               ANPRequestProcResult procResult)
            throws Exception
    {
        /* Check we have the required post-processing information */
        if (procResult.queryResult == null) {
            return ANPRequestProcessorAction.FAILED;
        }

        /* Generate the statement */
	ANPAuthnCapabilitiesStatement authnCapabStmt = 
            ANPAuthnCapabilitiesManager.getAuthnCapabilitiesStatement();

        /* Set the nonce, if any */
        ANPAuthnCapabQueryResult acqr = (ANPAuthnCapabQueryResult) procResult.queryResult;
        if (acqr.nonce != null) {
            authnCapabStmt.setNonce(acqr.nonce);
        }

        /* Generate the response */
        List set = new LinkedList();
        set.add(authnCapabStmt);
        response.setStatements(set);
        fillBasicResponse(null, request, response);

        /* Sign the response */
        /* To avoid unnecessary overhead, we only sign it if a nonce was
         * specified in the request, assuming it means that the client wants
         * specific protection of the Capabilities Statement
         */
        if (acqr.nonce != null) {
	    if (dir == null) {
                dir = ProviderDescriptorFactory.getInstance();
            }
            ProviderDescriptor us = dir.findLocal();
            response.setSignedBy(us.getProviderID());
            if (Config.debug) {
                System.err.println("signedBy set at AuthnCapabilities response");
            }
        }

        DebugUtils.debugTime("SimpleMessageANPRequestProcessor",
                             "Response to AuthnCapabilitiesQuery generated",
                             DebugUtils.LOW_LEVEL_FUNC);

        return ANPRequestProcessorAction.RETURN_RESPONSE;
    }


    private ANPRequestProcessorAction generateANPAuthnResponse(ANPRequest request,
					  ANPResponse response,
                                          ANPRequestProcResult procResult)
            throws Exception
    {
        /* Check we have the required post-processing information */
        if (procResult.queryResult == null) {
            return ANPRequestProcessorAction.FAILED;
        }
        ANPAuthnQueryResult aqr = (ANPAuthnQueryResult) procResult.queryResult;
        if (aqr.authnResult == null) {
            return ANPRequestProcessorAction.FAILED;
        }

        /* Generate the statement */
	ANPAuthnStatement authnStmt = new ANPAuthnStatement();
        authnStmt.setAuthnResult(aqr.authnResult);

        if (aqr.authnResult.equals("Redirect")) {
            if (aqr.redirectURL == null) {
                return ANPRequestProcessorAction.FAILED;
            }

            ANPRedirectInfo ri = new ANPRedirectInfo();
            ri.setRedirectURL(aqr.redirectURL);
            ri.setSessionInfoRequired(true);
            authnStmt.setRedirectInfo(ri);
        }

        /* Generate the response */
        List set = new LinkedList();
        set.add(authnStmt);
        response.setStatements(set);
        fillBasicResponse(null, request, response);

        DebugUtils.debugTime("SimpleMessageANPRequestProcessor",
                             "Response to AuthnQuery generated",
                             DebugUtils.LOW_LEVEL_FUNC);

        return ANPRequestProcessorAction.RETURN_RESPONSE;
    }


    private void fillBasicResponse(String failureCode, ANPRequest request,
                                  ANPResponse response) {
        SAMLPStatus status = new SAMLPStatus();
        if (failureCode != null) {
            status.setStatusCode(failureCode);
            if (Config.debug) {
                System.err.println("returning failureCode:" + failureCode);
            }
            if (failureCode.equals("samlp:VersionMismatch")) {
                response.setMajorVersion(1);
                response.setMinorVersion(0);
            } else {
                response.setMajorVersion(request.getMajorVersion());
                response.setMinorVersion(request.getMinorVersion());
            }
            if (Config.debug) {
                System.err.println("version set");
            }
        } else {
            if (Config.debug) {
                System.err.println("status code is samlp:Success");
            }
            status.setStatusCode("samlp:Success");
            response.setMajorVersion(request.getMajorVersion());
            response.setMinorVersion(request.getMinorVersion());
        }
        response.setStatus(status);
        if (Config.debug) {
            System.err.println("status set");
        }

        /* I prefer not to do it for every response, since it introduces overhead
	if (dir == null) {
            dir = ProviderDescriptorFactory.getInstance();
        }
        ProviderDescriptor us = dir.findLocal();
        response.setSignedBy(us.getProviderID());
        if (Config.debug) {
            System.err.println("signedBy set");
        }
	*/
    }
}
